# Skins

import slogan, slogan.pie
[false, true]= range(2)
__all__= slogan.autoPackage(__file__)

import string, math

## Defines a base class providing slogan's default skin.

# The default skin is a HTML 4.0/CSS solution to enable best redraw speed. The layout will not survive on older browsers; a table-based
# version could be derived from it.

class Skin:
  identity= 'default'
  def __init__(self):
    self.imgDir= slogan.html.config['imageUrl']+'skin/default/'

# pageType, outputPage

  def pageType(self, fvals):
    if fvals.get('t', '')=='i':
      return slogan.html.config['pieFormat'][1]
    if self.htmlLevel[:1]=='x':
      return 'text/xml'
    else:
      return 'text/html'

  def outputPage(self, stream, fvals):
    q= slogan.html.decodeQuery(fvals.get('q', ''))
    s= slogan.html.sortType(fvals.get('s', ''))
    d= slogan.html.displayType(fvals.get('d', ''))
    if fvals.get('t', '')=='i':
      results= slogan.query(q, s)
      slogan.pie.outputImage(stream, (q, s, d), slogan.pie.getSlices(results), slogan.html.config['pieFormat'][0])
    else:
      self.sequenceNumber= 0
      self.outputHead(stream, (q, s, d))
      self.outputBody(stream, (q, s, d))

# outputHead -- HTML <head> element

  def outputHead(self, stream, context):
    stream.write(slogan.html.doctypes[self.htmlLevel])
    stream.write('<html')
    if slogan.language.code!='':
      stream.write(' lang="'+slogan.language.code+'"')
    stream.write('><head>\n  <title>')
    self.outputTitle(stream, context)
    stream.write('</title>\n')
    if self.outputStyles!=None:
      stream.write('<style type="text/css">')
      if self.htmlLevel[:1]=='x':
        stream.write('<![CDATA[\n')
      else:
        stream.write('<!--\n')
      self.outputStyles(stream, context)
      if self.htmlLevel[:1]=='x':
        stream.write('  ]]>')
      else:
        stream.write('  /*-->')
      stream.write('</style>\n')
    if self.outputScripts!=None:
      stream.write('<script type="text/javascript">')
      if self.htmlLevel[:1]=='x':
        stream.write('<![CDATA[\n')
      else:
        stream.write('<!--\n')
      self.outputScripts(stream, context)
      if self.htmlLevel[:1]=='x':
        stream.write('  ]]>')
      else:
        stream.write('  //-->')
      stream.write('</script>\n')
    stream.write('</head>')

  htmlLevel= '4.0t'

# outputTitle -- document title text

  def outputTitle(self, stream, context):
    if len(context[0])==0:
      stream.write(slogan.html.encH(slogan.language.html_title_index))
    else:
      stream.write(slogan.html.encH(slogan.language.html_title_query))

# outputStyles -- CSS context. Set to None for no styles.

  def outputStyles(self, stream, context):
    stream.write('    body { background: rgb(224, 224, 224); color: black; margin: 0; padding: 0 }\n')
    stream.write('    a:link, a:visited { text-decoration: none; color: black }\n')
    stream.write('    a:hover { text-decoration: underline }\n')
    stream.write('    .logo, .left, .main { margin: 0; padding: 0 }\n')
    stream.write('    .logo, .left { position: absolute; top: 0; height: 100% } /* position: fixed better but not yet widely supported */\n')
    stream.write('    .logo { left: 0; width: 2em; background: url('+self.imgDir+'vblend.jpeg) repeat-x; color: white }\n')
    stream.write('    .left { left: 2em; width: 14em; margin-top: 8px }\n')
    stream.write('    .main { margin: 8px 1em 1em 17em }\n')
    stream.write('    h1 { text-align: center; padding: 8px 0; margin: 0 }\n')
    stream.write('    h2 { padding: 2px; background: url('+self.imgDir+'hblend.jpeg) repeat-y; color: white; border-top: solid 1px rgb(15, 46, 116); font-size: 100%; font-weight: normal; font-style: normal }\n')
    stream.write('    .left h2 { border-left: solid 1em rgb(15, 46, 116) }\n')
    stream.write('    .left .list { margin-left: 1em }\n')
    stream.write('    .table { padding: 1px; border: solid 2px; border-color: gray white white gray }\n')
    stream.write('    .rows { background: white; padding: 4px }\n')

# outputScripts -- JavaScript content. Set to None for no scripts.

  def outputScripts(self, stream, context):
    if context[2]=='tree':
      stream.write('    function toggle(subtree) {\n')
      stream.write('      if (document.getElementById && document.images) {\n')
      stream.write('        state= (document.getElementById("d"+subtree).style.display=="none");\n')
      stream.write('        document.getElementById("d"+subtree).style.display= state ? "block" : "none";\n')
      stream.write('        document.images["i"+subtree].src= "'+self.imgDir+'"+(state ? "close" : "open")+".gif";\n')
      stream.write('      }\n')
      stream.write('      return false;\n')
      stream.write('    }\n')

# outputBody -- main block-layout method

  def outputBody(self, stream, context):
    stream.write('<body>\n')
    stream.write('  <div class="logo">\n')
    self.outputLogo(stream, context)
    stream.write('  </div><div class="left">\n')
    self.outputTraits(stream, context)
    stream.write('  </div><div class="main">\n')
    if len(context[0])>0:
      self.outputCurrents(stream, context)
      self.outputControls(stream, context)
      stream.write('  </div><div class="main">\n')
      results= slogan.query(context[0], context[1])
      self.outputResults(stream, context, results)
    else:
      self.outputSummary(stream, context)
    stream.write('  </div>\n')
    stream.write('</body></html>')

# outputLogo

  def outputLogo(self, stream, context):
    stream.write('    <h1><img src="'+self.imgDir+'title.gif" width="15" height="162" alt="The Slogan Log Analyser"></h1>\n')

# outputTraits

  def outputTraits(self, stream, context):
    self.outputHeader(stream, slogan.language.html_traits)
    stream.write('    <table class="list" cellspacing="4" cellpadding="0" border="0">\n')
    l1= true
    for traitName in slogan.traitPriorities:
      trait= slogan.traitObjects[traitName]
      if trait.priority>=0:
        self.outputTrait(stream, context, trait)
    stream.write('    </table>\n')

# outputTrait

  def outputTrait(self, stream, context, trait):
    q= filter(lambda cond, identity= trait.identity: cond[0]!=identity, context[0])
    if len(q)>0 and len(q[-1][1])==0:
      q= q[:-1]
    q.append((trait.identity, []))
    stream.write('      <tr><td>')
    self.outputIcon(stream, trait.icon([]), (q, context[1], context[2]), trait.title)
    stream.write(' <a href="')
    self.outputLink(stream, (q, context[1], context[2]))
    stream.write('">'+slogan.html.encH(trait.title)+'</a></td></tr>\n')

# outputCurrents

  def outputCurrents(self, stream, context):
    self.outputHeader(stream, slogan.language.html_currents)
    stream.write('    <div class="table"><div class="rows"><table cellspacing="4" cellpadding="0" border="0">\n')
    q= []
    l1= true
    for condition in context[0]:
      q.append(condition)
      subparts= q[-1][1]
      q[-1]= (q[-1][0], [])
      self.outputCurrentTrait(stream, (q, context[1], context[2]))
      for subpart in subparts:
        q[-1][1].append(subpart)
        self.outputCurrentSubpart(stream, (q, context[1], context[2]))
    stream.write('    </table></div></div>\n')

# outputCurrentTrait

  def outputCurrentTrait(self, stream, context):
    trait= slogan.traitObjects[context[0][-1][0]]
    stream.write('      <tr><td>')
    self.outputIcon(stream, trait.icon([]), context, trait.title)
    stream.write(' <a href="')
    self.outputLink(stream, context)
    stream.write('">'+slogan.html.encH(trait.title)+'</a><td></tr>\n')

# outputCurrentSubpart

  def outputCurrentSubpart(self, stream, context):
    trait= slogan.traitObjects[context[0][-1][0]]
    stream.write('      <tr><td>')
    self.outputSpacer(stream)
    self.outputIcon(stream, trait.icon(context[0][-1][1]), context, trait.write(context[0][-1][1]))
    stream.write(' <a href="')
    self.outputLink(stream, context)
    stream.write('">'+slogan.html.encH(trait.write(context[0][-1][1]))+'</a></td></tr>\n')

# outputControls

  def outputControls(self, stream, context):
    stream.write('    <form method="get" action="'+slogan.html.config['queryUrl']+'"><p>\n')
    stream.write('      <label for="d">'+slogan.language.html_display+'</label>\n')
    stream.write('      <select name="d">\n')
    for d in slogan.html.displayTypes:
      if not (d=='pie' and not slogan.pie.available):
        stream.write('        <option value="'+slogan.html.encH(d)+'"')
        if d==context[2]:
          stream.write(' selected')
      stream.write('>'+slogan.html.encH(slogan.language.html_displays.get(d, d))+'</option>\n')
    stream.write('      </select>\n')
    stream.write('      <label for="s">'+slogan.language.html_sort+'</label>\n')
    stream.write('      <select name="s">\n')
    for s in slogan.html.sortTypes.keys():
      stream.write('        <option value="'+slogan.html.encH(s)+'"')
      if s==slogan.html.sortName(context[1]):
        stream.write(' selected')
      stream.write('>'+slogan.html.encH(slogan.language.html_sorts.get(s, s))+'</option>\n')
    stream.write('      </select>\n')
    stream.write('      <input type="hidden" name="q" value="'+slogan.html.encodeQuery(context[0])+'">\n')
    stream.write('      <input type="submit" value="'+slogan.language.html_change+'">\n')
    stream.write('    </p></form>\n')

# outputResults

  def outputResults(self, stream, context, results):
    self.outputHeader(stream, slogan.language.html_results)
    stream.write('    <div class="table"><div class="rows">\n')
    if len(results)==0:
      stream.write('        '+slogan.html.encH(slogan.language.html_nomatch)+'')
    else:
      largest= max(map(lambda x: x[1], results))
      total= reduce(lambda a, b: a+b, map(lambda x: x[1], results))

      if context[2]=='pie':
        results= self.outputResultsPie(stream, context, results)
        if len(results)>0:
          stream.write('      <h3><a name="other">'+slogan.html.encH(slogan.language.pie_other)+'</a></h3>\n')

      for result in results:
        if context[2]=='list':
          self.outputResultList(stream, context, result)
        elif context[2]=='tree':
          self.outputResultTree(stream, context, result, [])
        elif context[2]=='bar':
          self.outputResultBar(stream, context, result, (total, largest))
        elif context[2]=='pie':
          self.outputResultPie(stream, context, result, total)
    stream.write('    </div></div>\n')

# outputResultsPie -- put the pie image element in the HTML with a client-side image map

  def outputResultsPie(self, stream, context, results):
    def areaBbox(area, bbox):
      (x0, y0)= (area[0]-bbox[0], area[1]-bbox[1])
      (x1, y1)= (area[2]-bbox[0], area[3]-bbox[1])
      return string.join(map(lambda n: str(int(n)), [x0, y1, x1, y0]), ',')
    def areaPolygon(polygon, bbox):
      return string.join(map(lambda p, bbox=bbox: str(int(p[0]-bbox[0]))+','+str(int(p[1]-bbox[1])), polygon), ',')

    slices= slogan.pie.getSlices(results)
    (font, bbox, labels)= slogan.pie.placeLabels(context, slices)

    w= bbox[2]-bbox[0]
    h= bbox[3]-bbox[1]
    stream.write('    <div align="center"><img src="')
    self.outputLink(stream, context)
    stream.write(slogan.html.encH('&t=i'))
    stream.write('" width="'+str(w)+'" height="'+str(h)+'" alt="'+slogan.html.encH(slogan.language.html_displays['pie'])+'" usemap="#piemap" border="0"></div>\n')
    stream.write('    <map name="piemap">\n')

    for (val, text, icon, iconBbox, textBbox) in labels:
      for (isIcon, area) in [(true, iconBbox), (false, textBbox)]:
        stream.write('      <area shape="rect" coords="'+areaBbox(area, bbox)+'" alt="'+slogan.html.encH(text)+'" href="')
        if val==None:
          stream.write('#other')
        else:
          q= context[0][:]
          q[-1]= (q[-1][0], q[-1][1]+[val])
          self.outputLink(stream, (q, context[1], context[2]))
        stream.write('"')
        if isIcon:
          self.outputIconScript(stream, q)
        stream.write('>\n')

    pos= 0.0
    for (val, deg) in slices:
      nextPos= pos+deg
      steps= max(int(deg*75), 2)
      polygon= [(0.0, 0.0)]
      for p in map(lambda n, steps= steps: float(n)/(steps-1), range(steps)):
        rad= 2*math.pi*((nextPos*p + pos*(1.0-p))-0.25)
        polygon.append((slogan.pie.config['diameter']/2*math.cos(rad), slogan.pie.config['diameter']/2*math.sin(rad)))
      pos= nextPos
      stream.write('      <area shape="polygon" coords="'+areaPolygon(polygon,bbox)+'" alt="" href="')
      if val==None:
        stream.write('#other')
      else:
        q= context[0][:]
        q[-1]= (q[-1][0], q[-1][1]+[val])
        self.outputLink(stream, (q, context[1], context[2]))
      stream.write('">\n')

    stream.write('    </map>\n')
    resultsDone= map(lambda s: s[0], slices)
    return filter(lambda r, d= resultsDone: r[0] not in d, results)

# outputResultList

  def outputResultList(self, stream, context, result):
    q= context[0][:]
    trait= slogan.traitObjects[q[-1][0]]
    if result[0]!='':
      q[-1]= (q[-1][0], q[-1][1][:]+[result[0]])
    n= trait.write(q[-1][1])
    stream.write('        <table cellpadding="2" cellspacing="0" border="0"><tr valign="top"><td>')
    self.outputIcon(stream, trait.icon(q[-1][1]), (q, context[1], context[2]), n)
    stream.write('</td><td width="100%"><a href="')
    self.outputLink(stream, (q, context[1], context[2]))
    stream.write('">'+slogan.html.encH(n)+'</a></td><td align="right">')
    stream.write(repr(long(result[1]))[:-1])
    stream.write('</td></tr></table>\n')

# outputResultBar

  def outputResultBar(self, stream, context, result, (total, largest)):
    q= context[0][:]
    trait= slogan.traitObjects[q[-1][0]]
    if result[0]!='':
      q[-1]= (q[-1][0], q[-1][1][:]+[result[0]])
    n= trait.write(q[-1][1])
    pc= round(result[1]*1000/total)/10
    bs= int(result[1]*200/largest)
    stream.write('        <table cellpadding="2" cellspacing="0" border="0"><tr valign="top"><td>')
    self.outputIcon(stream, trait.icon(q[-1][1]), (q, context[1], context[2]), n)
    stream.write('</td><td width="200">')
    if bs>0:
      stream.write('<img src="'+self.imgDir+'bar.gif" width="'+str(bs)+'" height="16" alt="'+'*'*(bs/20)+'">')
    if bs<200:
      stream.write('<img src="'+self.imgDir+'spacer.gif" width="'+str(200-bs)+'" height="16" alt="">')
    stream.write('</td><td width="100%"><a href="')
    self.outputLink(stream, (q, context[1], context[2]))
    stream.write('">'+slogan.html.encH(n)+'</a></td><td align="right">')
    stream.write(str(pc)+'%')
    stream.write('</td></tr></table>\n')

# outputResultPie -- print the "other" results not plotted on the pie chart

  def outputResultPie(self, stream, context, result, total):
    q= context[0][:]
    trait= slogan.traitObjects[q[-1][0]]
    if result[0]!='':
      q[-1]= (q[-1][0], q[-1][1][:]+[result[0]])
    n= trait.write(q[-1][1])
    stream.write('        <table cellpadding="2" cellspacing="0" border="0"><tr valign="top"><td>')
    self.outputIcon(stream, trait.icon(q[-1][1]), (q, context[1], context[2]), n)
    stream.write('</td><td width="100%"><a href="')
    self.outputLink(stream, (q, context[1], context[2]))
    stream.write('">'+slogan.html.encH(n)+'</a></td><td align="right">')
    stream.write(str(round(result[1]*1000.0/total)/10))
    stream.write('%</td></tr></table>\n')

# outputResultTree

  def outputResultTree(self, stream, context, result, indents):
    thisId= str(self.sequenceNumber)
    self.sequenceNumber= self.sequenceNumber+1
    q= context[0][:]
    trait= slogan.traitObjects[q[-1][0]]
    if result[0]!='':
      q[-1]= (q[-1][0], q[-1][1][:]+[result[0]])
    n= trait.write(q[-1][1])
    stream.write('        <table cellpadding="2" cellspacing="0" border="0"><tr valign="top"><td width="'+str((len(indents)+2)*20)+'">')
    for i in range(len(indents)):
      stream.write(('<img src="'+self.imgDir))
      if i<len(indents)-1:
        stream.write(['treel', 'spacer'][indents[i]])
      else:
        stream.write(['treer', 'treeb'][indents[i]])
      stream.write('.gif" width="20" height="20" alt="" align="middle">')
    self.outputIcon(stream, trait.icon(q[-1][1]), (q, context[1], context[2]), n)
    if result[0]!='':
      subresults= slogan.query(q, context[1])
    else:
      subresults= []
    if len(subresults)==1 and subresults[0][0]=='':
      subresults= []
    if len(subresults)>0:
      stream.write('<a href="#d'+thisId+'" onclick="return toggle(\''+thisId+'\');" ondblclick="return toggle(\''+thisId+'\');"><img id="i'+thisId+'" src="'+self.imgDir+'open.gif" align="middle" border="0"></a>')
    else:
      stream.write('<img src="'+self.imgDir+'spacer.gif" width="20" height="20" alt="" align="middle">')
    stream.write('</td><td width="100%"><a href="')
    self.outputLink(stream, (q, context[1], context[2]))
    stream.write('">'+slogan.html.encH(n)+'</a></td><td align="right">')
    stream.write(str(long(result[1]))[:-1])
    stream.write('</td>')
    if len(indents)!=0:
      stream.write('<td width="'+str(len(indents)*20)+'">'+('<img src="'+self.imgDir+'spacer.gif" width="20" height="20" alt="" align="middle">')*len(indents)+'</td>')
    stream.write('</tr></table>\n')
    if len(subresults)>0:
      stream.write('        <div id="d'+thisId+'" style="display: none">\n')
      for i in range(len(subresults)):
        subresult= subresults[i]
        subindents= indents+[(i==len(subresults)-1)]
        self.outputResultTree(stream, (q, context[1], context[2]), subresult, subindents)
      stream.write('        </div>\n')

# outputSummary

  def outputSummary(self, stream, context):
    stream.write('<p>Hello. This is Slogan. Why not click on some of those traits on the left, yes that might do something.</p>')

# outputHeader

  def outputHeader(self, stream, text):
    stream.write('  <h2>')
    stream.write(slogan.html.encH(text))
    stream.write('</h2>\n')

# outputIcon

  def outputIcon(self, stream, icon, context, alt):
    stream.write('<a href="')
    self.outputLink(stream, context)
    stream.write('"')
    self.outputIconScript(stream, context[0])
    stream.write('><img src="'+slogan.html.encH(slogan.html.findIcon(icon)[0])+'" width="20" height="20" ')
    stream.write('align="middle" border="0" alt="'+slogan.html.encH(alt)+'"></a>')

# outputIconScript

  def outputIconScript(self, stream, q):
    pass

# outputLink

  def outputLink(self, stream, context):
    qs= slogan.html.queryString({'q': slogan.html.encodeQuery(context[0]), 's': slogan.html.sortName(context[1]), 'd': context[2]})
    stream.write(slogan.html.encH(slogan.html.config['queryUrl']+qs))

# outputSpacer

  def outputSpacer(self, stream):
    stream.write('<img src="'+self.imgDir+'spacer.gif" width="20" height="20" align="middle" alt="">')

# xmlt -- if running in XHTML mode, use terminators in empty tags

  def xmlt(self):
    return ['', ' /'][self.htmlLevel[:1]=='x']
