# BrowserTrait

import slogan, slogan.traits
import string, re
[false, true]= range(2)

uaRe= re.compile('((?:\(.*?\))|(?:.*?)) +')

wellKnownPlatforms= [
  ('windows 95', 'Win95'),
  ('windows 98', 'Win98'),
  ('windows me', 'WinME'),
  ('windows nt 5.0', 'Win2000'),
  ('windows nt', 'WinNT'),
  ('windows 3.1', 'Win3.1'),
  ('win16', 'Win3.1'),
  ('win32', 'Win'),
  ('windows', 'Win'),
  ('macintosh', 'Mac'),
  ('apple', 'Mac'),
  ('risc os', 'RISC OS'),
  ('acorn', 'RISC OS'),
  ('beos', 'BeOS'),
  ('amiga', 'Amiga'),

  ('linux', 'X11 (Linux)'),
  ('solaris', 'X11 (Solaris)'),
  ('sunos', 'X11 (Solaris)'),
  ('hp-ux', 'X11 (HP-UX)'),
  ('hpux', 'X11 (HPUX)'),
  ('irix', 'X11 (IRIX)'),
  ('tru64', 'X11 (Tru64)'),
  ('digital', 'X11 (Tru64)'),
  ('bsd', 'X11 (BSD)'),
  ('aix', 'X11 (AIX)'),
  ('x11', 'X11'),
]

def containsOnly(text, chars):
  for c in text:
    if c not in chars:
      return false
  return true

# Trait for the User-Agent header. The trait is made up of browser, version and platform,
# but extracting this information is tricky because nobody agrees how to format
# User-Agent lines, and everybody wants to be bloody Mozilla.

class Trait(slogan.traits.Trait):
  identity= 'browser'
  extended= 'cs(User-Agent)'
  priority= 60
  size= [32, 16, 16]
  def __init__(self):
    self.title= slogan.language.browser_title

  def read(self, logpart):
    browser= ''
    version= ''
    platform= ''

# Find a (comment) part amonst the space-separated tokens. Special case for
# Opera - check to see if it follows the comment

    products= uaRe.findall(logpart+' ')
    for i in range(len(products)):
      uaPart= products[i]
      if uaPart[:1]=='(' and uaPart[-1:]==')':
        comment= uaPart[1:-1]
        if products[i+1:i+2]==['Opera']:
          products[:i+1]= []
        else:
          products[i:i+1]= []
        break
    else:
      comment= ''

# Look to see if there's a comment starting 'compatible...'; if so, grab the real browser
# name and version from the comment.

    commentParts= map(string.strip, string.split(comment, ';'))
    if len(commentParts)>=2 and string.lower(commentParts[0])=='compatible':
      productParts= string.split(commentParts[1], '/', 1)
      if len(productParts)==2:
        [browser, version]= productParts
      else:
        productParts= string.split(productParts[0], ' ', 1)
        if len(productParts)==2:
          [browser, version]= productParts
        else:
          browser= productParts[0]

# Otherwise, get the browser name from the first product token. If there's no obvious version
# number embedded in it, look at the next product token and see if it might be a version
# number

    elif len(products)>=1:
      productParts= string.split(products[0], '/', 1)
      if len(productParts)==2:
        [browser, version]= productParts
      else:
        browser= productParts[0]
        if len(products)>=2 and containsOnly(products[1], 'v.0123456789'):
          version= products[1]

# If the browser name in the product token was Mozilla, but there wasn't a 'compatible'
# string, check the version number. If it's 5+, assume the UA string conforms to the
# mozilla.org UA standard and look for the real browser name in the other tokens.
# Otherwise, assume Netscape, and attempt to remove version number branding

      if browser=='Mozilla':
        platform= commentParts[0]
        if len(commentParts)>=3:
          platform= platform+' ('+string.split(commentParts[2], ' ', 1)[0]+')'

        try:
          v= int(version)
        except ValueError:
          v= 0
        if v>=5:
          for product in products[1:]:
            productParts= string.split(products[1], '/', 1)
            if string.lower(productParts[0]) not in ['mozilla', 'gecko']:
              if len(productParts)==2:
                [browser, version]= productParts
              else:
                browser= productParts[0]
              break
        else:
          browser= 'Netscape'
          for i in range(len(version)):
            if version[i] not in 'v.012345678':
              version= version[:i]
              break

# Okay... if we didn't manage to read a definitive platform name, do a last-ditch
# search through the whole string for some well-known ones.

    if version!='' and platform=='':
      allua= string.lower(logpart)
      for (find, name) in wellKnownPlatforms:
        if string.find(allua, find)!=-1:
          platform= name
          break

    return [browser, version, platform]

  def write(self, partlist):
    if len(partlist)>0 and partlist[0]!='':
      return string.join(partlist)
    return slogan.language.browser_none
